@@ -4,7 +4,7 @@
 Aplicativo para Gerenciar a PNADC.
 
 Rafael Guerreiro Osorio - Ipea
-2020-21
+2020-21-22
 """
 
 
@@ -67,16 +67,12 @@ _VQUES = 'quesito'  # número da questão para variáveis do questionário
 _MISS = 'vazio'  # chave para "não aplicável" em _VCAT
 
 # constantes para status de arquivos
-ATUALIZADO = 'sincronizado'
-REMOVER = 'REMOVER'
-DOWNLOAD = 'BAIXAR'
-UNKNOWN = 'verificar atualizações'
-UNLOCAL = 'ignorar arquivo local'
-UNREMOTE = 'ignorar arquivo remoto'
+ATUALIZADO = 'Sincronizado'
+BAIXAR = 'Baixar'
+IGNORAR = 'Ignorar'
+REMOVER = 'Remover'
 UNCHECK = {ATUALIZADO: REMOVER,
-           REMOVER: UNLOCAL,
-           DOWNLOAD: UNREMOTE,
-           UNKNOWN: REMOVER}
+           BAIXAR: IGNORAR}
 
 # delimitador de arquivos CSV
 DELIMITER = ','
@@ -127,7 +123,10 @@ class MainWindow(QMainWindow):
         self.ultatu = None
         self.ultver = None
         self.copialocal = None
+        self.local = []
+        self.locold = []
         self.remote = []
+        self.remold = []
         self.elementos = []
         self.msgbox = QMessageBox()
         self._ui_reset()
@@ -173,11 +172,11 @@ class MainWindow(QMainWindow):
             item.setData(0, Qt.UserRole, elemento)
             if child['folder']:
                 item.setIcon(0, icofld)
-                # item.setExpanded(elemento['expanded'])
+                item.setExpanded(elemento['expanded'])
             else:
                 item.setIcon(0, icodoc)
-                if child['check'] == 0:
-                    item.setText(1, UNCHECK[child['status']])
+                if child['check'] == Qt.Unchecked:
+                    item.setText(1, IGNORAR)
             return item
 
         icofld = QIcon(str(Path(APP_PATH, 'folder.svg')))
@@ -247,14 +246,33 @@ class MainWindow(QMainWindow):
         # das checkboxes de parents em tristate
         item.treeWidget().setCurrentItem(None)
 
-    def _tree_data(self, files):
-        """Gera lista de elementos para tree a partir de file list."""
+    def _tree_data(self):
+        """Gera lista de elementos para tree a partir de self.remote."""
+        def status(file):
+            options = {}
+            if file in self.locold:
+                options['check'] = Qt.Checked
+                if file in self.local:
+                    options['status'] = ATUALIZADO
+                    options['expanded'] = False
+                else:
+                    options['status'] = BAIXAR
+                    options['expanded'] = True
+            elif file not in self.locold and file not in self.remold:
+                options['check'] = Qt.Checked
+                options['status'] = BAIXAR
+                options['expanded'] = True
+            elif file not in self.locold and file in self.remold:
+                options['check'] = Qt.Unchecked
+                options['status'] = BAIXAR
+                options['expanded'] = False
+            return options
+
         parentid = {'.': -1}
         elementos = []
         uniqueid = 0
-        for filerec in files:
-            file = filerec[0]
-            options = filerec[1]
+        for file in self.remote:
+            options = status(file)
             parentes = [str(par) for par
                         in pPath(file['parent'], file['name']).parents
                         if str(par) != '.']
@@ -315,13 +333,11 @@ class MainWindow(QMainWindow):
         arquivos = []
         for elemento in self.elementos[:]:
             if not elemento['folder']:
-                arquivos.append(
-                    [{'parent': elemento['file']['parent'],
-                      'name': elemento['file']['name'],
-                      'size': elemento['file']['size']},
-                     {'status': elemento['status'],
-                      'check': elemento['check'],
-                      'expanded': elemento['expanded']}])
+                arquivos.append({'parent': elemento['file']['parent'],
+                                 'name': elemento['file']['name'],
+                                 'size': elemento['file']['size'],
+                                 'status': elemento['status'],
+                                 'check': elemento['check']})
         return arquivos
 
     def _menu_csv(self):
@@ -373,7 +389,11 @@ class MainWindow(QMainWindow):
             self.copialocal = file
             with ZipFile(file, 'w', ZIP_DEFLATED) as target:
                 target.writestr('pynad.json',
-                                json.dumps([None, None, []]))
+                                json.dumps({'ultatu': None,
+                                            'ultver': None,
+                                            'local': [],
+                                            'remote': []}))
+            self.local, self.locold, self.remold = [], [], []
             self.ui.copia_local_nome.setText(Path(self.copialocal).name)
             ultima = ('Última atualização: <b>nenhuma</b>')
             self.ui.copia_local_ultatu.setText(ultima)
@@ -381,7 +401,7 @@ class MainWindow(QMainWindow):
             self.ui.menu_sync.setEnabled(True)
             self.ui.menu_sync_verify.setEnabled(True)
             if self.remote:
-                self._tree_data(self.remote)
+                self._tree_data()
                 self._tree_build()
                 self.ui.menu_sync_update.setEnabled(True)
             else:
@@ -394,7 +414,7 @@ class MainWindow(QMainWindow):
             try:
                 with ZipFile(self.copialocal) as source:
                     pynadjson = json.loads(source.read('pynad.json'))
-                    if isinstance(pynadjson, list) and len(pynadjson) == 3:
+                    if isinstance(pynadjson, dict):
                         result = True
                     else:
                         result = False
@@ -438,7 +458,7 @@ class MainWindow(QMainWindow):
                 mensagem = ('A cópia local da Pnadc contém os paineis:<br>'
                             + f'<b>{", ".join(panlist)}</b>'
                             + '<br><br>O processamento dos painéis'
-                            + ' é lento e exige 16GB de memória RAM.'
+                            + ' é lento e exige ao menos 8GB de memória RAM.'
                             + '<br><br>Continuar?')
                 resposta = self.msgbox.question(self, 'Confirmar',
                                                 mensagem,
@@ -493,6 +513,7 @@ class MainWindow(QMainWindow):
         # serão removidos
         remover = [painel for painel in atuais if painel not in paineis]
         atualizar = []
+
         for painel in paineis:
             if painel in atuais:
 
@@ -524,7 +545,7 @@ class MainWindow(QMainWindow):
         mensagem = ('<b>O arquivo com painéis da Pnadc está desatualizado.</b>'
                     + rem + atu + ign
                     + '<br><br>O processamento dos painéis'
-                    + ' é lento e exige 16GB de memória RAM.'
+                    + ' é lento e exige ao menos 8GB de memória RAM.'
                     + '<br><br>Continuar?')
         resposta = self.msgbox.question(self, 'Confirmar',
                                         mensagem,
@@ -552,20 +573,43 @@ class MainWindow(QMainWindow):
     def _menu_update(self):
         """Sincroniza a cópia local."""
         # primeiro, verificar se há updates - de acordo com as seleções
-        curfiles = [ele for ele in self.elementos
-                    if not ele['folder']]
+        self.local = self._list_local_files()
+        treeselection = self._tree_sel_export()
         actions = {}
-        for file in curfiles:
+        treefiles = []
+        for file in [file for file in treeselection
+                     if file['check'] == Qt.Checked]:
             status = file['status']
-            if not file['check']:
-                status = UNCHECK[status]
             if status not in actions:
                 actions[status] = [0, 0, []]
             actions[status][0] += 1
-            actions[status][1] += file['file']['size']
+            actions[status][1] += file['size']
             actions[status][2].append(str(pPath('originais',
-                                                file['file']['parent'],
-                                                file['file']['name'])))
+                                                file['parent'],
+                                                file['name'])))
+            treefiles.append({'parent': file['parent'],
+                              'name': file['name'],
+                              'size': file['size']})
+        ignorarlst = [file for file in treeselection
+                      if file['check'] == Qt.Unchecked]
+        if ignorarlst:
+            actions[IGNORAR] = [0, 0, []]
+            for file in ignorarlst:
+                actions[IGNORAR][0] += 1
+                actions[IGNORAR][1] += file['size']
+                actions[IGNORAR][2].append(str(pPath('originais',
+                                                     file['parent'],
+                                                     file['name'])))
+        removerlst = [file for file in self.local
+                      if file not in treefiles]
+        if removerlst:
+            actions[REMOVER] = [0, 0, []]
+            for file in removerlst:
+                actions[REMOVER][0] += 1
+                actions[REMOVER][1] += file['size']
+                actions[REMOVER][2].append(str(pPath('originais',
+                                               file['parent'],
+                                               file['name'])))
         atualtxt = ''
         ignortxt = ''
         removtxt = ''
@@ -573,20 +617,18 @@ class MainWindow(QMainWindow):
         if ATUALIZADO in actions:
             atualtxt += f'{actions[ATUALIZADO][0]:4.0f} arquivos atualizados'
             atualtxt += f' - {sizestr(actions[ATUALIZADO][1])}<br>'
-        if UNLOCAL in actions:
-            ignortxt += f'{actions[UNLOCAL][0]:4.0f} arquivos locais ignorados'
-            ignortxt += f' - {sizestr(actions[UNLOCAL][1])}<br>'
-        if UNREMOTE in actions:
-            ignortxt += f'{actions[UNREMOTE][0]:4.0f} arquivos'
+        if IGNORAR in actions:
+            ignortxt += f'{actions[IGNORAR][0]:4.0f} arquivos'
             ignortxt += ' remotos ignorados'
-            ignortxt += f' - {sizestr(actions[UNREMOTE][1])}<br>'
+            ignortxt += f' - {sizestr(actions[IGNORAR][1])}<br>'
+        if BAIXAR in actions:
+            downltxt += f'<b>{BAIXAR:>7}</b> {actions[BAIXAR][0]:4.0f}'
+            downltxt += f' arquivos - {sizestr(actions[BAIXAR][1])}<br>'
         if REMOVER in actions:
             removtxt += f'<b>{REMOVER:>7}</b> {actions[REMOVER][0]:4.0f}'
             removtxt += f' arquivos - {sizestr(actions[REMOVER][1])}<br>'
-        if DOWNLOAD in actions:
-            downltxt += f'<b>{DOWNLOAD:>7}</b> {actions[DOWNLOAD][0]:4.0f}'
-            downltxt += f' arquivos - {sizestr(actions[DOWNLOAD][1])}<br>'
-        if not any(status in actions for status in (REMOVER, DOWNLOAD)):
+
+        if not any(status in actions for status in (REMOVER, BAIXAR)):
             mensagem = f'{atualtxt}{ignortxt}<br>'
             mensagem += 'Sem arquivos para baixar ou remover'
             self.msgbox.warning(self, 'Nada a atualizar', mensagem)
@@ -619,35 +661,26 @@ class MainWindow(QMainWindow):
 
             # baixar arquivos
             if downltxt:
-                self._proc_downloads(actions[DOWNLOAD])
+                self._proc_downloads(actions[BAIXAR])
 
             # atualizar o status dos arquivos
-            locais = self._list_local_files()
-            arvore = self._tree_sel_export()
-            finais = []
-            for item in arvore:
-                for file in locais:
-                    if file[0] == item[0]:
-
-                        # existe em remote e em locais - atualizado
-                        if any(file[0] == rem[0] for rem in self.remote):
-                            item[1]['status'] = ATUALIZADO
-                        else:
-
-                            # existe em locais mas não em remote - remover
-                            item[1]['status'] = REMOVER
-
-                # existe em locais ou em remote - entra
-                if (any(item[0] == file[0] for file in locais)
-                        or any(item[0] == rem[0] for rem in self.remote)):
-                    item[0]['parent'] = str(item[0]['parent'])
-                    finais.append(item)
+            self.local = self._list_local_files()
 
             # escrever novo pynad.json
+            # pathlib (parent) nao e compativel com json
+            self.ultatu = time.time()
+            local = [{'parent': str(file['parent']),
+                      'name': file['name'],
+                      'size': file['size']} for file in self.local]
+            remot = [{'parent': str(file['parent']),
+                      'name': file['name'],
+                      'size': file['size']} for file in self.remote]
             with ZipFile(self.copialocal, 'a', ZIP_DEFLATED) as target:
                 target.writestr('pynad.json',
-                                json.dumps([time.time(), self.ultver,
-                                            finais]))
+                                json.dumps({'ultatu': self.ultatu,
+                                            'ultver': self.ultver,
+                                            'local': local,
+                                            'remote': remot}))
             self._proc_copialocal()
             self._proc_metadata()
             if self.ui.menu_tools_csv.isChecked():
@@ -666,7 +699,6 @@ class MainWindow(QMainWindow):
             mensagem = ('Mapear os arquivos da Pnadc em '
                         + '<b>ftp.ibge.gov.br</b> pode demorar alguns minutos'
                         + '<br><br>Verificar agora?')
-
         resposta = self.msgbox.question(self, 'Confirmar',
                                         mensagem,
                                         QMessageBox.Yes |
@@ -674,9 +706,7 @@ class MainWindow(QMainWindow):
                                         QMessageBox.No)
         if resposta == self.msgbox.No:
             return
-        self._ui_enabled(False)
         self._list_remote_files()
-        self._ui_enabled(True)
         if self.remote:
             self.ultver = time.time()
             dtu = time.localtime(self.ultver)
@@ -714,12 +744,8 @@ class MainWindow(QMainWindow):
         pynadinfo.seek(0)
         info = json.loads(pynadinfo.read())
 
-        # json é incompatível com Path, caminho foi guardado como string
-        for item in info[2]:
-            item[0]['parent'] = pPath(item[0]['parent'])
-
-        # 1o elemento de info é a data da atualização do arquivo
-        self.ultatu = info[0]
+        # data da atualização do arquivo
+        self.ultatu = info['ultatu']
         if self.ultatu:
             dtu = time.localtime(self.ultatu)
             ultima = ('Última atualização:'
@@ -730,14 +756,11 @@ class MainWindow(QMainWindow):
             ultima = ('Última atualização: <b>nenhuma</b>')
         self.ui.copia_local_ultatu.setText(ultima)
 
-        # 2o elemento é a data da última verificação que corresponde
-        # à atualização, mas só carrega se não houver verificação atual
-        # a verificação realizada na seção prepondera sobre a da última
-        # atualização - mas podem ter arquivos abertos diferentes, aí
-        # fica o mapa mais novo
-        if self.ultver is None or (info[1] is not None
-                                   and self.ultver < info[1]):
-            self.ultver = info[1]
+        # data da última verificação dos arquivos no IBGE
+        # corresponde a lista de arquivos remotos armazenada em info
+        if self.ultver is None or (info['ultver'] is not None
+                                   and self.ultver < info['ultver']):
+            self.ultver = info['ultver']
 
         # ainda pode ser None - arquivos novos sem verificação/atualização
         if self.ultver:
@@ -749,17 +772,19 @@ class MainWindow(QMainWindow):
                                           'Status na cópia local'])
             self.ui.menu_sync_update.setEnabled(True)
 
-        # construir a lista de arquivos
-        info[2].extend([remote for remote in self.remote
-                        if all(remote[0] != file[0] for file in info[2])])
+        self.locold = info['local']
+        self.remold = info['remote']
 
-        # marcar arquivos obsoletos
-        for file in info[2]:
-            if self.remote and all(file[0] != rem[0] for rem in self.remote):
-                file[1]['status'] = REMOVER
+        # json é incompatível com Path, caminhos guardados como string
+        for filelst in (self.locold, self.remold):
+            for idx, _file in enumerate(filelst):
+                filelst[idx]['parent'] = pPath(filelst[idx]['parent'])
 
-        # construir a árvore de arquivos
-        self._tree_data(info[2])
+        # a árvore mostra sempre os arquivos remotos da ver. mais recente
+        if not self.remote and self.remold:
+            self.remote = self.remold
+        self._list_local_files()
+        self._tree_data()
         self._tree_build()
 
     def _proc_csv(self):
@@ -852,6 +877,7 @@ class MainWindow(QMainWindow):
         metadata = TextIOWrapper(original, encoding='utf-8')
         metadata.seek(0)
         meta = json.loads(metadata.read())
+
         dicvars = [(var,
                     meta[var][_VPOS][0] - 1,
                     meta[var][_VPOS][1]) for var in meta]
@@ -864,20 +890,71 @@ class MainWindow(QMainWindow):
         with ZipFile(BytesIO(archive.read(microdado[0]))) as src:
             original = BytesIO()
             original.write(src.read(src.namelist()[0]))
-
-        # arquivo csv na memória
-        tgt = StringIO()
-        tgt.write(DELIMITER.join([var[0] for var in dicvars]) + '\n')
         fixwidth = TextIOWrapper(original, encoding='utf-8')
         size = fixwidth.tell()
         fixwidth.seek(0)
+
+        """
+        ALTERAÇÃO PNADC 2021
+        inclusão de pesos de bootstrap nas bases
+        são iguais para as pessoas de domicílios
+
+        v1028XXX
+        v1032XXX
+
+        onde XXX vai de 1 a 200
+
+        solução - arquivo separado com registro por domicilio
+        indexado por upa v1008 v1014
+        """
+
+        # nomes das variáveis de peso BS
+        if 'visita' in nome:
+            pesosbs = ['v1032' + f'{x:03.0f}' for x in range(1, 201)]
+        else:
+            pesosbs = ['v1028' + f'{x:03.0f}' for x in range(1, 201)]
+
+        # se tiver pesos BS no arquivo, estão em dicvars
+        # criar arquivo para armazená-los separadamente
+        # FALTA INCLUIR IDENTIFCADORES PARA MERGE
+        if len(dicvars) > len([var for var in dicvars
+                               if var[0] not in pesosbs]):
+            pesosbs = [var for var in dicvars if var[0] in pesosbs]
+            dicvars = [var for var in dicvars if var not in pesosbs]
+            pesosbs = [(var, meta[var][_VPOS][0] - 1,
+                        meta[var][_VPOS][1]) for var in
+                       ('upa', 'v1008', 'v1014')] + pesosbs
+        else:
+            pesosbs = []
+
+        # arquivos csv na memória
+        tgt = StringIO()
+        tgt.write(DELIMITER.join([var[0] for var in dicvars]) + '\n')
+        if pesosbs:
+            tgtpeso = StringIO()
+            tgtpeso.write(DELIMITER.join([var[0] for var in pesosbs]) + '\n')
+
         message = self._ui_show_prgs(f'{message[0]} Gerando {nome} (0%)')
         report = 1
         read = 0
+        controle = 0
         for reg in fixwidth:
             csvreg = DELIMITER.join([reg[var[1]:var[2]].strip(' .')
                                      for var in dicvars])
             tgt.write(csvreg + '\n')
+
+            # se tiver peso, registro é por domicílio
+            # a pnadc vem ordenada por domicilio
+            if pesosbs:
+                campos = [reg[var[1]:var[2]].strip(' .') for var in pesosbs]
+                iddom = (int(campos[0]) * 1000
+                         + int(campos[1]) * 10 + int(campos[0]))
+                if iddom != controle:
+                    controle = iddom
+                    csvreg = DELIMITER.join(campos)
+                    tgtpeso.write(csvreg + '\n')
+
+            # atualiza statusbar
             read += len(reg)
             if (read / size) * 100 >= report:
                 message = self._ui_show_prgs(f'{message[0]}'
@@ -890,6 +967,14 @@ class MainWindow(QMainWindow):
                                      + ' - aguarde...')
         tgt.seek(0)
         archive.writestr(str(pPath(MICRO, nome)), tgt.read())
+        if pesosbs:
+            message = self._ui_show_prgs(f'{message[0]}'
+                                         + f' Compactando pesosBS.{nome}'
+                                         + ' - aguarde...')
+            tgtpeso.seek(0)
+            archive.writestr(str(pPath(MICRO,
+                                       f'pesosBS.{nome}')),
+                             tgtpeso.read())
 
     def _proc_downloads(self, downloads):
         """Baixa arquivos de ftp.ibge.gov.br."""
@@ -911,6 +996,9 @@ class MainWindow(QMainWindow):
                            + f' arquivo {file} de {downloads[0]} -'
                            + f' {Path(download).name}')
                 message = self._ui_show_prgs(message)
+
+                # TODO incluir try para se der timeout no retrbinary
+                # timeout: timed out
                 with archive.open(download, 'w') as target:
                     server.retrbinary(f'RETR {remotefile}', target.write)
                 size += [file.file_size for file in archive.infolist()
@@ -1038,9 +1126,10 @@ class MainWindow(QMainWindow):
             partes = Path(fileinfo.filename).parts
             if root == partes[0]:
                 arquivos.append(
-                    [{'parent': pPath(*partes[1:-1]),
-                      'name': partes[-1],
-                      'size': fileinfo.file_size}])
+                    {'parent': pPath(*partes[1:-1]),
+                     'name': partes[-1],
+                     'size': fileinfo.file_size})
+        self.local = arquivos
         return arquivos
 
     def _list_remote_files(self):
@@ -1072,12 +1161,10 @@ class MainWindow(QMainWindow):
                         + f'{len(remotefiles)} arq'
                         + f'uivos em {elapsed(start)}')
                     QGuiApplication.processEvents()
-                    remotefiles.append([{'parent': pPath(pnad, ftpcur),
+                    remotefiles.append({'parent': pPath(pnad, ftpcur),
                                          'name': item,
-                                         'size': server.size(str(sourcename))},
-                                        {'status': DOWNLOAD,
-                                         'check': Qt.Checked,
-                                         'expanded': False}])
+                                         'size': server.size(str(sourcename))})
+        self._ui_enabled(False)
         try:
             start = time.time()
             stub = '  Verificando arquivos da Pnadc em ftp.ibge.gov.br: '
@@ -1097,6 +1184,7 @@ class MainWindow(QMainWindow):
             remotefiles = []
         self.remote = remotefiles
         self.ui.statusbar.showMessage('')
+        self._ui_enabled(True)
 
     def _panels_available(self):
         """
@@ -1135,40 +1223,36 @@ class MainWindow(QMainWindow):
             pid = ano * 10 + tri
 
             # verificar continuidade dos microdados trimestrais
-            continuidade = True
             for seq in range(1, 5):
                 proximo = microdados[PNADCT][idx + seq]
                 anoprox = int(Path(proximo[0]).name.split('_')[1][2:6])
                 triprox = int(Path(proximo[0]).name.split('_')[1][:2])
                 if tri + seq <= 4:
                     if triprox != tri + seq or ano != anoprox:
-                        continuidade = False
                         break
                 else:
                     if triprox != (tri + seq - 4) or anoprox != (ano + 1):
-                        continuidade = False
                         break
 
-            if continuidade:
-                panels[pid] = [[item[0] for item
-                                in microdados[PNADCT][idx:idx + 5]]]
-                if PNADCA in microdados:
-                    anuais = []
-                    for seq in range(5):
-                        if tri > 4:
-                            tri = 1
-                            ano += 1
-                        anuais.append((str(ano), f'trimestre{tri}'))
-                        anuais.append((str(ano), f'visita{seq + 1}'))
-                        tri += 1
-                    panels[pid].append([microdado[0] for microdado
-                                        in microdados[PNADCA]
-                                        if any(f'_{anual[0]}_'
-                                               in microdado[0] and
-                                               anual[1] in microdado[0]
-                                               for anual in anuais)])
-                else:
-                    panels[pid].append([])
+            panels[pid] = [[item[0] for item
+                            in microdados[PNADCT][idx:idx + 5]]]
+            if PNADCA in microdados:
+                anuais = []
+                for seq in range(5):
+                    if tri > 4:
+                        tri = 1
+                        ano += 1
+                    anuais.append((str(ano), f'trimestre{tri}'))
+                    anuais.append((str(ano), f'visita{seq + 1}'))
+                    tri += 1
+                panels[pid].append([microdado[0] for microdado
+                                    in microdados[PNADCA]
+                                    if any(f'_{anual[0]}_'
+                                           in microdado[0] and
+                                           anual[1] in microdado[0]
+                                           for anual in anuais)])
+            else:
+                panels[pid].append([])
         return panels
 
     def _panels_dwelling(self, panels, archive, microdados=None):
@@ -1559,7 +1643,7 @@ class MainWindow(QMainWindow):
                           'pidgrp', 'pidgrpent', 'pidind', 'pidcla',
                           'pidindent', 'piddnd', 'piddnm', 'piddna')
             variaveis = [var for var in pesheader
-                         if var[:2] in ('v1', 'v2') or var[:3] == 'vd2'
+                         if var[:2] in ('v1', 'v2')
                          and var not in constantes]
             variaveis.sort()
             variaveis = ['ano', 'trimestre', 'pidpeso'] + variaveis
@@ -1699,6 +1783,7 @@ class altFTP(FTP):
 
 def pynad_app():
     """Inicia o app."""
+    print('nova')
     app = QApplication([])
     gui = MainWindow()
     gui.show()
